#include <stdio.h>
#include <stdlib.h>
#include "lrz.h"

// Program entry point
int main(int argc, char **argv) {
    unsigned char *src, *dst;
    int srclen, dstlen, x;
    FILE *fPtr;

    // Check the command line arguments
    if (argc != 3)
        { printf("Usage: %s <infile> <outfile>\n", argv[0]); return 0; }

    // Get the size of the file
    fPtr = fopen(argv[1], "rb");
    if (fPtr == NULL)
        { printf("Could not open %s\n", argv[1]); return 1; }
    fseek(fPtr, 0, SEEK_END);
    srclen = ftell(fPtr);

    // Check the file size for errors
    if (!srclen) {
        fclose(fPtr);
        printf("%s contains no data\n", argv[1]);
        return 1;
    }

    // Load the file data
    fseek(fPtr, 0, SEEK_SET);
    src = malloc(srclen);
    x = fread(src, 1, srclen, fPtr);
    close(fPtr);
    if (x != srclen) { printf("Error reading %s\n", argv[1]); return 1; }

    // Get the uncompressed size of the file
    dstlen = lrzGetSize(src, srclen);
    if (dstlen < 1) {
        free(src);
        printf("%s contains errors\n", argv[1]);
        return 1;
    }

    // Decode the file
    dst = malloc(dstlen);
    x = lrzDecode(src, srclen, dst, dstlen);
    if (x != LRZ_SUCCESS) {
        free(dst); free(src);
        printf("%s contains errors\n", argv[1]);
        return 1;
    }

    // Save the decoded output to disk
    fPtr = fopen(argv[2], "wb");
    if (fPtr == NULL) {
        free(dst); free(src);
        printf("Cannot write to %s\n", argv[2]);
        return 1;
    }
    fwrite(dst, 1, dstlen, fPtr);
    fclose(fPtr);

    // User output
    printf("Wrote to %s\n", argv[2]);

    // Clean up and exit
    free(dst);
    free(src);
    return 0;
}
